
From: Dmitry Torokhov <dtor_core@ameritech.net>

  Driver core: add default driver attributes to struct bus_type
  
Signed-off-by: Dmitry Torokhov <dtor@mail.ru>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/drivers/base/bus.c     |   37 +++++++++++++++++++++++++++++++++++--
 25-akpm/include/linux/device.h |    1 +
 2 files changed, 36 insertions(+), 2 deletions(-)

diff -puN drivers/base/bus.c~input-add-bus-default-driver-attributes drivers/base/bus.c
--- 25/drivers/base/bus.c~input-add-bus-default-driver-attributes	2004-06-27 22:51:10.722564984 -0700
+++ 25-akpm/drivers/base/bus.c	2004-06-27 22:51:10.729563920 -0700
@@ -415,7 +415,7 @@ static int device_add_attrs(struct bus_t
 static void device_remove_attrs(struct bus_type * bus, struct device * dev)
 {
 	int i;
-	
+
 	if (bus->dev_attrs) {
 		for (i = 0; attr_name(bus->dev_attrs[i]); i++)
 			device_remove_file(dev,&bus->dev_attrs[i]);
@@ -471,6 +471,37 @@ void bus_remove_device(struct device * d
 	}
 }
 
+static int driver_add_attrs(struct bus_type * bus, struct device_driver * drv)
+{
+	int error = 0;
+	int i;
+
+	if (bus->drv_attrs) {
+		for (i = 0; attr_name(bus->drv_attrs[i]); i++) {
+			error = driver_create_file(drv, &bus->drv_attrs[i]);
+			if (error)
+				goto Err;
+		}
+	}
+ Done:
+	return error;
+ Err:
+	while (--i >= 0)
+		driver_remove_file(drv, &bus->drv_attrs[i]);
+	goto Done;
+}
+
+
+static void driver_remove_attrs(struct bus_type * bus, struct device_driver * drv)
+{
+	int i;
+
+	if (bus->drv_attrs) {
+		for (i = 0; attr_name(bus->drv_attrs[i]); i++)
+			driver_remove_file(drv, &bus->drv_attrs[i]);
+	}
+}
+
 
 /**
  *	bus_add_driver - Add a driver to the bus.
@@ -499,6 +530,7 @@ int bus_add_driver(struct device_driver 
 		driver_attach(drv);
 		up_write(&bus->subsys.rwsem);
 
+		driver_add_attrs(bus, drv);
 	}
 	return error;
 }
@@ -516,6 +548,7 @@ int bus_add_driver(struct device_driver 
 void bus_remove_driver(struct device_driver * drv)
 {
 	if (drv->bus) {
+		driver_remove_attrs(drv->bus, drv);
 		down_write(&drv->bus->subsys.rwsem);
 		pr_debug("bus %s: remove driver %s\n", drv->bus->name, drv->name);
 		driver_detach(drv);
@@ -610,7 +643,7 @@ static int bus_add_attrs(struct bus_type
 static void bus_remove_attrs(struct bus_type * bus)
 {
 	int i;
-	
+
 	if (bus->bus_attrs) {
 		for (i = 0; attr_name(bus->bus_attrs[i]); i++)
 			bus_remove_file(bus,&bus->bus_attrs[i]);
diff -puN include/linux/device.h~input-add-bus-default-driver-attributes include/linux/device.h
--- 25/include/linux/device.h~input-add-bus-default-driver-attributes	2004-06-27 22:51:10.724564680 -0700
+++ 25-akpm/include/linux/device.h	2004-06-27 22:51:10.730563768 -0700
@@ -56,6 +56,7 @@ struct bus_type {
 
 	struct bus_attribute	* bus_attrs;
 	struct device_attribute	* dev_attrs;
+	struct driver_attribute	* drv_attrs;
 
 	int		(*match)(struct device * dev, struct device_driver * drv);
 	struct device * (*add)	(struct device * parent, char * bus_id);
_
